@@ -22,6 +22,8 @@ from glanceclient.openstack.common import strutils
 from glanceclient.v2 import schemas
 
 DEFAULT_PAGE_SIZE = 20
+SORT_DIR_VALUES = ('asc', 'desc')
+SORT_KEY_VALUES = ('created_at', 'namespace')
 
 
 class NamespaceController(object):
@@ -88,8 +90,17 @@ class NamespaceController(object):
 
     def list(self, **kwargs):
         """Retrieve a listing of Namespace objects
-
-        :param page_size: Number of namespaces to request in each request
+        :param page_size: Number of items to request in each paginated request
+        :param limit: Use to request a specific page size. Expect a response
+                      to a limited request to return between zero and limit
+                      items.
+        :param marker: Specifies the namespace of the last-seen namespace.
+                       The typical pattern of limit and marker is to make an
+                       initial limited request and then to use the last
+                       namespace from the response as the marker parameter
+                       in a subsequent limited request.
+        :param sort_key: The field to sort on (for example, 'created_at')
+        :param sort_dir: The direction to sort ('asc' or 'desc')
         :returns generator over list of Namespaces
         """
 
@@ -129,6 +140,25 @@ class NamespaceController(object):
         else:
             filters['limit'] = kwargs['page_size']
 
+        if 'marker' in kwargs:
+            filters['marker'] = kwargs['marker']
+
+        sort_key = kwargs.get('sort_key')
+        if sort_key is not None:
+            if sort_key in SORT_KEY_VALUES:
+                filters['sort_key'] = sort_key
+            else:
+                raise ValueError('sort_key must be one of the following: %s.'
+                                 % ', '.join(SORT_KEY_VALUES))
+
+        sort_dir = kwargs.get('sort_dir')
+        if sort_dir is not None:
+            if sort_dir in SORT_DIR_VALUES:
+                filters['sort_dir'] = sort_dir
+            else:
+                raise ValueError('sort_dir must be one of the following: %s.'
+                                 % ', '.join(SORT_DIR_VALUES))
+
         for param, value in six.iteritems(filters):
             if isinstance(value, list):
                 filters[param] = strutils.safe_encode(','.join(value))
